package  {
	
	import com.adobe.flashplayer10demo.model.Locale;
	import com.adobe.flashplayer10demo.ui.Style;
	
	import fl.controls.ComboBox;
	import fl.controls.UIScrollBar;
	import fl.data.DataProvider;
	
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.SampleDataEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundTransform;
	import flash.utils.ByteArray;
	
	public class SoundDemo extends Sprite {
		
		
		
		//Constants
		protected const SOUND_MULTIPLYER:uint = 8;
		protected const DEMO_WIDTH:Number = 792;
		protected const DEMO_HEIGHT:Number = 390;
		
		// Public Properties:
		public var wave:Sprite;
		public var canvas:Sprite;
		public var soundChooser:ComboBox;
		
		// Protected Properties:
		protected var lastVolume:Number;
		protected var dynamicSound:Sound;
		protected var activeSoundChannel:SoundChannel;
		protected var mp3sounds:Array;
		protected var toolTip:MovieClip;
		protected var dynamicSoundSamples:ByteArray;
		protected var lastMouseX:Number;
		protected var lastMouseY:Number;
		protected var currentSoundIndex:Number = 1;
		protected var frameCount:Number = 0;
		protected var trackMousePosistion:Boolean = false;
		
		// Initialization:
		public function SoundDemo() { configUI(); }
		
		/**
		* Called from the demo shell when this demo is ready to play.
		*
		*/
		public function animationComplete():void {
			canvas.addEventListener(Event.ENTER_FRAME, animate, false, 0, true);
			activeSoundChannel = mp3sounds[currentSoundIndex].play();
		}
		
		/**
		* Pause the current playing sound, and stop the animation.
		*
		*/
		public function pause():void {
			if (activeSoundChannel) {
				lastVolume = activeSoundChannel.soundTransform.volume;
				var st:SoundTransform = new SoundTransform(0);
				activeSoundChannel.soundTransform = st;
			}
			
			if (dynamicIsSelected()) { canvas.removeEventListener(MouseEvent.CLICK, onCanvasClick); }
			
			canvas.removeEventListener(Event.ENTER_FRAME, animate);
		}
		
		/**
		* Resume the current sound and start the animation.
		*
		*/
		public function resume():void {
			lastVolume = isNaN(lastVolume) ? 1 : lastVolume;
			
			if (activeSoundChannel) {
				var st:SoundTransform = new SoundTransform(lastVolume);
				activeSoundChannel.soundTransform = st;
			}
			
			if (dynamicIsSelected()) { canvas.addEventListener(MouseEvent.CLICK, onCanvasClick, false, 0, true); }
			
			canvas.addEventListener(Event.ENTER_FRAME, animate, false, 0, true);
		}
		
		protected function onLocaleLoad(p_event:Event):void { populateLocaleLabels(); }
		
		/**
		 * After the locale has been loaded populate all the labels.
		 * 
		 */
		protected function populateLocaleLabels():void {
			var dp:DataProvider = new DataProvider();
			
			dp.addItem({label:Locale.getString('SOUND_CHOOSE_SOUND')});
			dp.addItem({label:Locale.getString('SOUND_ROCK_GUITAR')});
			dp.addItem({label:Locale.getString('SOUND_VOICE')});
			dp.addItem({label:Locale.getString('SOUND_ALTERNATIVE')});
			dp.addItem({label:Locale.getString('SOUND_SLOW_GUITAR')});
			dp.addItem({label:Locale.getString('SOUND_SOOTHING')});
			dp.addItem({label:Locale.getString('SOUND_GENERATED')});
			
			soundChooser.dataProvider = dp;
			soundChooser.rowCount = dp.length;
			soundChooser.selectedIndex = currentSoundIndex;
			soundChooser.drawNow();
		}
		
		protected function generateDynamicSound(p_event:SampleDataEvent):void {
			//var time:Number = p_event.position / 44.1; //Milliseconds since the sound started.
			
			/*
			Set the number of dynamicSoundSamples.
			The minimum is 512, but causes processor hit, maximum 8192 faster to process.
			*/
			
			for (var c:int=0; c<8192; c++) {
				p_event.data.writeFloat(calculateDynamicSoundChannel(c));
				p_event.data.writeFloat(calculateDynamicSoundChannel(c));
				
				//Use this code to create a simple sine wave.
				/*
					var sample:Number = Math.sin((Number(c+p_event.position)/Math.PI/2))*0.25;
					p_event.data.writeFloat(sample);
					p_event.data.writeFloat(sample);
				*/
			}
		}
		
		protected function animate(p_event:Event) {
			// Limit the drawing to every 4 frames.
			if (++frameCount % 4 != 1) { return; }
			frameCount = 5;
			
			if (currentSoundIndex == 0 || currentSoundIndex >= mp3sounds.length) { return; }
			
			var sampleLength:Number = 1777;
			
			var left:Number;
			var right:Number;
			var len:Number;
			
			//Clean up the old sound samples.
			dynamicSoundSamples.clear();
			dynamicSoundSamples.position = 0; //.clear() doesn't 0 the posistion. -- Bug #234883 - Bug was deffered.
			
			//Figure out how much to read from the sound stream.
			if (dynamicIsSelected()) {
				len = 1000;
			} else {
				len = mp3sounds[currentSoundIndex].extract(dynamicSoundSamples, sampleLength);
				// create a seamless loop
				if (len < sampleLength) {
					len += mp3sounds[currentSoundIndex].extract(dynamicSoundSamples, sampleLength-len, 0);
				}
			}
			
			//Remove the old wave.
			wave.graphics.clear();
			wave.graphics.lineStyle(1.0, 0xaaccee);
			
			dynamicSoundSamples.position = 0;
			
			//Set up variables reused in the loop, for speed reasons.
			var dynamicSelected:Boolean = dynamicIsSelected();
			var amp:Number = .1;
			
			var lineX:Number;
			var lineY:Number;
			
			for (var c:int = 0; c<len; c++) {
				if (dynamicSelected) {
					//If the current sound is dynamic, use the same formula that plays the sound.
					right = left = calculateDynamicSoundChannel(c);
				} else {
					//Else just read from the sound channel.
					left = dynamicSoundSamples.readFloat();
					right = dynamicSoundSamples.readFloat();
				}
				lineX = c/(len/DEMO_WIDTH);
				lineY = left*200;
				wave.graphics.lineTo(lineX, lineY);
			}
		}
		
		protected function onCanvasClick(p_event:MouseEvent):void {
			trackMousePosistion = !trackMousePosistion;
			
			if (trackMousePosistion == true) {
				toolTip.x = stage.stageWidth + 10;
				addChild(toolTip);
				canvas.addEventListener(MouseEvent.MOUSE_MOVE, setMousePositions, false, 0, true);
			} else {
				if (contains(toolTip)) {  removeChild(toolTip); }
				canvas.removeEventListener(MouseEvent.MOUSE_MOVE, setMousePositions);
			}
		}
		
		protected function setMousePositions(p_event:MouseEvent):void {
			lastMouseX = canvas.mouseX;
			lastMouseY = canvas.mouseY;
			
			toolTip.label.text = 'x: ' + lastMouseX + 'px \ny: ' + lastMouseY + ' px';
			toolTip.x = lastMouseX;
			toolTip.y = lastMouseY;
		}
		
		//Used to calculate the dynamic sound channel, and to draw the visuals.
		protected function calculateDynamicSoundChannel(p_offset:Number):Number {
			return Math.sin((Number(p_offset)/Math.PI/(lastMouseX/canvas.height*SOUND_MULTIPLYER)))*(lastMouseY/canvas.width*2);
		}
		
		protected function dynamicIsSelected():Boolean {
			return (soundChooser as ComboBox).selectedLabel == Locale.getString('SOUND_GENERATED');
		}
		
		protected function changeAudio(p_event:Event) {
			if (activeSoundChannel) { activeSoundChannel.stop(); }
			
			if (contains(toolTip)) { removeChild(toolTip); }
			
			currentSoundIndex = p_event.target.selectedIndex;
			
			if (currentSoundIndex == 0) {
				pause();
				return;
			}
			
			resume();
			
			if (dynamicIsSelected()) {
				canvas.addEventListener(MouseEvent.CLICK, onCanvasClick, false, 0, true);
				onCanvasClick(null);
				dynamicSound.addEventListener(SampleDataEvent.SAMPLE_DATA, generateDynamicSound, false, 0, true);
				addChild(toolTip);
			} else {
				dynamicSound.removeEventListener(SampleDataEvent.SAMPLE_DATA, generateDynamicSound);
				canvas.removeEventListener(MouseEvent.CLICK, onCanvasClick);
			}
			
			activeSoundChannel = mp3sounds[currentSoundIndex].play(0, 0);
		}
		
		protected function configUI():void {
			dynamicSound = new Sound();
			dynamicSound.addEventListener(SampleDataEvent.SAMPLE_DATA, generateDynamicSound, false, 0, true);
			
			mp3sounds = [];
			
			//When we sample the dynamic sound, we use this ByteArray to hold the temporary sound data.
			dynamicSoundSamples = new ByteArray();
			
			toolTip = new ToolTip();
			
			wave.y = DEMO_HEIGHT>>1;
			wave.x = 0;
			
			//Pull in embedded MP3 files.
			mp3sounds.push('');
			mp3sounds.push(new track1());
			mp3sounds.push(new track2());
			mp3sounds.push(new track3());
			mp3sounds.push(new track4());
			mp3sounds.push(new track5());
			mp3sounds.push(dynamicSound);
			
			Style.getInstance();
			Style.setInstanceStyle(soundChooser.dropdown.verticalScrollBar, UIScrollBar)
			
			//If we are standalone (not loaded into the main shell), so just play the sound.  Else we wait for the shell to call animationComplete();
			if (parent is Stage) {
				animationComplete();
			}
			
			soundChooser.addEventListener(Event.CHANGE, changeAudio, false, 0, true);
			
			setMousePositions(null);
			
			canvas.addChild(wave);
			addChild(soundChooser); //Always on top
			
			Locale.getInstance().addEventListener(Event.COMPLETE, onLocaleLoad, false, 0, true);
			Locale.languageCode = null;
			Locale.getInstance().load();
			
		}
	}
}